@@ -26,17 +26,6 @@ extern int r_curl_is_postfields_option(CURLoption x);
 #define HAS_CURLOPT_EXPECT_100_TIMEOUT_MS 1
 #endif
 
-char R_WINDOWS_CA_BUNDLE[MAX_PATH];
-
-SEXP R_set_bundle(SEXP path){
-  strcpy(R_WINDOWS_CA_BUNDLE, CHAR(asChar(path)));
-  return mkString(R_WINDOWS_CA_BUNDLE);
-}
-
-SEXP R_get_bundle(){
-  return mkString(R_WINDOWS_CA_BUNDLE);
-}
-
 int total_handles = 0;
 
 void clean_handle(reference *ref){
@@ -104,7 +93,7 @@ static int xferinfo_callback(void *clientp, xftype dltotal, xftype dlnow, xftype
 }
 
 /* Set default headers here, these are only allocated once */
-static struct curl_slist * default_headers(){
+static struct curl_slist * default_headers(void){
   static struct curl_slist * headers = NULL;
   if(headers == NULL){
     headers = curl_slist_append(headers, "Expect:");
@@ -134,19 +123,15 @@ static void set_handle_defaults(reference *ref){
 
   /* Only set a default CA bundle for openssl */
   #ifdef _WIN32
+  curl_easy_setopt(handle, CURLOPT_SSL_OPTIONS, CURLSSLOPT_NO_REVOKE);
   struct curl_tlssessioninfo *tlsinfo = NULL;
   if(curl_easy_getinfo(handle, CURLINFO_TLS_SSL_PTR, &tlsinfo) == CURLE_OK){
     if(tlsinfo->backend == CURLSSLBACKEND_OPENSSL) {
       const char *ca_bundle = getenv("CURL_CA_BUNDLE");
       if(ca_bundle != NULL) {
         curl_easy_setopt(handle, CURLOPT_CAINFO, ca_bundle);
-      } else if( R_WINDOWS_CA_BUNDLE != NULL && strlen(R_WINDOWS_CA_BUNDLE)){
-        /* on windows a cert bundle is included with R version 3.2.0 */
-        curl_easy_setopt(handle, CURLOPT_CAINFO, R_WINDOWS_CA_BUNDLE);
       } else {
-        /* disable cert validation for older versions of R */
-        curl_easy_setopt(handle, CURLOPT_SSL_VERIFYHOST, 0L);
-        curl_easy_setopt(handle, CURLOPT_SSL_VERIFYPEER, 0L);
+        curl_easy_setopt(handle, CURLOPT_SSL_OPTIONS, CURLSSLOPT_NO_REVOKE | CURLSSLOPT_NATIVE_CA);
       }
     }
   }
@@ -191,11 +176,6 @@ static void set_handle_defaults(reference *ref){
   /* dummy readfunction because default can freeze R */
   assert(curl_easy_setopt(handle, CURLOPT_READFUNCTION, dummy_read));
 
-  /* seems to be needed for native WinSSL */
-#ifdef _WIN32
-  curl_easy_setopt(handle, CURLOPT_SSL_OPTIONS, CURLSSLOPT_NO_REVOKE);
-#endif
-
   /* set default progress printer (disabled by default) */
 #ifdef HAS_XFERINFOFUNCTION
   assert(curl_easy_setopt(handle, CURLOPT_XFERINFOFUNCTION, xferinfo_callback));
@@ -210,13 +190,13 @@ static void set_handle_defaults(reference *ref){
 #endif
 }
 
-SEXP R_new_handle(){
+SEXP R_new_handle(void){
   reference *ref = calloc(1, sizeof(reference));
   ref->refCount = 1;
   ref->handle = curl_easy_init();
   total_handles++;
   set_handle_defaults(ref);
-  SEXP prot = PROTECT(allocVector(VECSXP, 5)); //for protecting callback functions
+  SEXP prot = PROTECT(allocVector(VECSXP, 6)); //for protecting callback functions
   SEXP ptr = PROTECT(R_MakeExternalPtr(ref, R_NilValue, prot));
   R_RegisterCFinalizerEx(ptr, fin_handle, TRUE);
   setAttrib(ptr, R_ClassSymbol, mkString("curl_handle"));
@@ -314,6 +294,14 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
         (curl_debug_callback) R_curl_callback_debug));
       assert(curl_easy_setopt(handle, CURLOPT_DEBUGDATA, val));
       SET_VECTOR_ELT(prot, 4, val); //protect gc
+    } else if (key == CURLOPT_SSL_CTX_FUNCTION){
+      if (TYPEOF(val) != CLOSXP)
+        error("Value for option %s (%d) must be a function.", optname, key);
+
+      assert(curl_easy_setopt(handle, CURLOPT_SSL_CTX_FUNCTION,
+                              (curl_ssl_ctx_callback) R_curl_callback_ssl_ctx));
+      assert(curl_easy_setopt(handle, CURLOPT_SSL_CTX_DATA, val));
+      SET_VECTOR_ELT(prot, 5, val); //protect gc
     } else if (key == CURLOPT_URL) {
       /* always use utf-8 for urls */
       const char * url_utf8 = translateCharUTF8(STRING_ELT(val, 0));
@@ -336,9 +324,12 @@ SEXP R_handle_setopt(SEXP ptr, SEXP keys, SEXP values){
       }
       assert(curl_easy_setopt(handle, key, (curl_off_t) asReal(val)));
     } else if(r_curl_is_postfields_option(key) || r_curl_is_string_option(key)){
+      if(key == CURLOPT_POSTFIELDS){
+        key = CURLOPT_COPYPOSTFIELDS;
+      }
       switch (TYPEOF(val)) {
       case RAWSXP:
-        if(key == CURLOPT_POSTFIELDS || key == CURLOPT_COPYPOSTFIELDS)
+        if(key == CURLOPT_COPYPOSTFIELDS)
           assert(curl_easy_setopt(handle, CURLOPT_POSTFIELDSIZE_LARGE, (curl_off_t) Rf_length(val)));
         assert(curl_easy_setopt(handle, key, RAW(val)));
         break;
@@ -447,7 +438,7 @@ SEXP make_rawvec(unsigned char *ptr, size_t size){
   return out;
 }
 
-SEXP make_namesvec(){
+SEXP make_namesvec(void){
   SEXP names = PROTECT(allocVector(STRSXP, 7));
   SET_STRING_ELT(names, 0, mkChar("url"));
   SET_STRING_ELT(names, 1, mkChar("status_code"));
@@ -485,6 +476,50 @@ SEXP R_get_handle_response(SEXP ptr){
   return make_handle_response(ref);
 }
 
-SEXP R_total_handles(){
+SEXP R_get_handle_speed(SEXP ptr){
+  CURL *handle = get_handle(ptr);
+#ifdef USE_CURL_OFF_T
+  curl_off_t dl = 0;
+  curl_off_t ul = 0;
+  curl_easy_getinfo(handle, CURLINFO_SPEED_DOWNLOAD_T, &dl);
+  curl_easy_getinfo(handle, CURLINFO_SPEED_UPLOAD_T, &ul);
+#else
+  double dl = 0;
+  double ul = 0;
+  curl_easy_getinfo(handle, CURLINFO_SPEED_DOWNLOAD, &dl);
+  curl_easy_getinfo(handle, CURLINFO_SPEED_UPLOAD, &ul);
+#endif
+  SEXP out = Rf_allocVector(REALSXP, 2);
+  REAL(out)[0] = (double) dl;
+  REAL(out)[1] = (double) ul;
+  return out;
+}
+
+SEXP R_get_handle_clength(SEXP ptr){
+  CURL *handle = get_handle(ptr);
+#ifdef USE_CURL_OFF_T
+  curl_off_t cl = 0;
+  curl_easy_getinfo(handle, CURLINFO_CONTENT_LENGTH_DOWNLOAD_T, &cl);
+#else
+  double cl = 0;
+  curl_easy_getinfo(handle, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &cl);
+#endif
+  return Rf_ScalarReal((double) cl < 0 ? NA_REAL : cl);
+}
+
+SEXP R_get_handle_received(SEXP ptr){
+  CURL *handle = get_handle(ptr);
+#ifdef USE_CURL_OFF_T
+  curl_off_t dl = 0;
+  curl_easy_getinfo(handle, CURLINFO_SIZE_DOWNLOAD_T, &dl);
+#else
+  double dl = 0;
+  curl_easy_getinfo(handle, CURLINFO_SIZE_DOWNLOAD, &dl);
+#endif
+  return Rf_ScalarReal((double) dl);
+}
+
+
+SEXP R_total_handles(void){
   return(ScalarInteger(total_handles));
 }
